//--------------------------------------------------------------//
// oscil~.c - MSP Oscillator - Cooper Baker - (c)2007			//
//																//
//	[tab] = 4													//
//																//
//	Ideal Function Generated Waveform							//
//																//
//	Arguments: Frequency										//
//	Default:	  0												//
//--------------------------------------------------------------//

#include	<Carbon/Carbon.h>
#include	<MaxAPI/MaxAPI.h>
#include	<MaxAudioAPI/MaxAudioAPI.h>
#include	<math.h>

void *oscil_class;					// pointer to this 'class'

// oscil~ structure ------------------------------------------------------ 
typedef struct _oscil				// oscil data structure
{ 
	t_pxobject	x_obj;				// every msp external needs this; t_pxobject struct contains object info
	double		phase;				// wave playback phase
	double		in_phase;			// input phase
	double		in_freq;			// input frequency
	double		freq;				// wave frequency
	float		s_rate;				// sampling rate
	int			conn_sw;			// connection switch for dsp block
} t_oscil; 

// prototypes ------------------------------------------------------------------ 
void *oscil_new( double frequency );					// makes a new oscil~ object
void oscil_free(t_oscil *x);							// deletes the oscil~ object
void oscil_dsp(t_oscil *x, t_signal **sp, short *count);// sets up dsp for oscil
t_int *oscil_perform(t_int *w);							// the dsp perform function
void oscil_assist(t_oscil *x, void *b, long m, long a, char *s); // inlet / outlet assistance
static inline float smoothwave(t_oscil*x){return(sin((x->phase)*TWOPI));}
														// ideal sine function	
// main ------------------------------------------------------------------------
void main(void) 
{ 
    setup((t_messlist **)&oscil_class, (method)oscil_new, (method)oscil_free, (short)sizeof(t_oscil), 0L, A_DEFFLOAT, 0); 
    addmess((method)oscil_dsp, "dsp", A_CANT, 0);		// add dsp method
   	addmess((method)oscil_assist, "assist", A_CANT, 0);	// assist method 
    dsp_initclass();									// init the class
	post( "" );								
	post( "cb.oscil~ :: ideal function generated sinewave oscillator" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );}										// post after init completes

// oscil_dsp --------------------------------------------------------------------
void oscil_dsp(t_oscil *x, t_signal **sp, short *count)	// oscil_dsp(*data struct, **signal struct, signal count )
{ 
	if( count[0] && count[1] )
		x->conn_sw = 3;									// both connected
	else if( !count[0] && count[1] )
		x->conn_sw = 2;									// right connected
	else if( count[0] && !count[1] )
		x->conn_sw = 1;									// left connected
	else
		x->conn_sw = 0;									// nothing connected


	x->s_rate = sp[0]->s_sr;							// sampling rate
	dsp_add( oscil_perform, 5, sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n, x );
}						 // 5 = number of values passed to oscil_perform

// oscil_perform ---------------------------------------------------------------- 
t_int *oscil_perform(t_int *w)							// oscil_perform runs once per signal vector
{
	t_float *in1	= ( t_float *)( w[1] );				// inlet signal vector location
	t_float *in2	= ( t_float *)( w[2] );
	t_float *out	= ( t_float *)( w[3] );				// outlet signal vector location
	int n			= ( int )( w[4] );					// vector size in samples
	t_oscil *x		= (t_oscil *)(	w[5] );				// oscil struct location
	
// DSP BLOCK ///////////////////////////////////////////////////////////////////
	while ( n-- )										// while vector exists...
	{ 
		x->in_freq = *in1++;
		x->in_freq = ( x->in_freq > 0 ) ? x->in_freq : 0; // constrain frequency
		x->in_phase = *in2++;

		while( x->in_phase >= 1 )						// wrap phase
		{
			x->in_phase = x->in_phase - 1;
		}
		while( x->in_phase < 0 )						// wrap phase
		{
			x->in_phase = x->in_phase + 1;
		}
		
		switch( x->conn_sw )
		{
			default:									
			case 0:	x->phase += x->freq / x->s_rate;	// increment phase
					break;
			case 1: x->freq = x->in_freq;				// update frequency
					x->phase += x->freq / x->s_rate;	// increment phase
					break;
			case 2:	x->phase = x->in_phase;				// phase is right input
					break;
			case 3:	x->freq = x->in_freq;				// update frequency
					x->phase += x->freq / x->s_rate + x->in_phase;	// increment phase
					break;
		}

		*out++ = smoothwave( x );						// calculate the sample
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 6);		// always return w plus one more than the 2nd argument in dsp_add() 
} 

// oscil_assist -----------------------------------------------------------------
void oscil_assist(t_oscil *x, void *b, long m, long a, char *s)
{
	switch (m)											// inlet / outlet switch
	{
		case 1:											// 1 = inlets
			switch (a)									// inlet number switch
			{
				case 0:									// inlet 0
					sprintf(s, "(signal) Frequency");	// assist message
					break;
				case 1:									// inlet 1
					sprintf(s, "(signal) Phase");		// assist message
					break;
				default:
					break;
			}
			break;
		case 2:											// 2 = outlets
			switch (a)									// outlet number switch
			{
				case 0:									// outlet 0
					sprintf(s, "(signal) Waveform");	// assist message
					break;
				default:
					break;
			}
			break;
	}
}

// oscil_new() ------------------------------------------------------------------ 
void *oscil_new( double frequency )
{														// oscil~ 'constructor'
	int i;												// increment var
	
	t_oscil *x = (t_oscil *)newobject(oscil_class);		// register the object with max and save its address as *x
	dsp_setup((t_pxobject *)x, 2);						// create inlets 
	outlet_new((t_pxobject *)x, "signal");				// create outlet 

	x->freq = frequency;
	
	return (x);											// where am i?
} 

// oscil_free() -----------------------------------------------------------------
void oscil_free(t_oscil *x)								// oscil~ 'destructor'
{
	dsp_free((t_pxobject *)x);							// msp free function
}
