/* -----------------------------------------------------------------------------
 *
 *	spiker~
 *		turns signals into single sample spikes at located at each wave peak
 *		
 *
 *	2007 . Cooper Baker . www.cooperbaker.com
 *
 *----------------------------------------------------------------------------*/

#include <Carbon/Carbon.h>
#include <MaxAPI/MaxAPI.h>
#include <MaxAudioAPI/MaxAudioAPI.h>

void *spiker_class;										// pointer to this 'class'

// spiker~ structure ------------------------------------------------------ 
typedef struct _spiker									// spiker data structure
{ 
    t_pxobject x_obj;									// every max external needs this; t_pxobject struct contains object info
	Float32		sample[3];								//
	Float32		spike;
} t_spiker; 

// prototypes ------------------------------------------------------------------ 
void *spiker_new(void);									// makes a new spiker~ object
void spiker_free(t_spiker *x);							// deletes the spiker~ object
void spiker_dsp(t_spiker *x, t_signal **sp, short *count);	// sets up dsp for spiker
t_int *spiker_perform(t_int *w);						// the dsp perform function
void spiker_assist(t_spiker *x, void *b, long m, long a, char *s);
														// inlet / outlet assistance
// main ------------------------------------------------------------------------
void main(void) 
{ 
	setup((t_messlist **)&spiker_class, (method)spiker_new, (method)spiker_free, (short)sizeof(t_spiker), 0L, 0); 
	addmess((method)spiker_dsp, "dsp", A_CANT, 0);		// add dsp method
	addmess((method)spiker_assist, "assist", A_CANT, 0);// assist method 
	dsp_initclass();	
	post( "" );								
	post( "cb.spiker~ :: turns signals into single sample spikes at located at each wave peak" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );
}														// post after init completes

// spiker_dsp --------------------------------------------------------------------
void spiker_dsp(t_spiker *x, t_signal **sp, short *count)// spiker_dsp(*data struct, **signal struct, signal count )
{ 
	dsp_add( spiker_perform, 4, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n, x );
}						  // 4 = number of values passed to spiker_perform

// spiker_perform ---------------------------------------------------------------- 
t_int *spiker_perform(t_int *w)							// spiker_perform runs once per signal vector
{ 
	t_float *in	= (t_float *)(w[1]);					// inlet signal vector location
	t_float *out	= (t_float *)(w[2]);				// outlet signal vector location
	int n		= (int)(w[3]);							// vector size in samples
	t_spiker *x	= (t_spiker *)(w[4]);					// spiker struct location
	
	int i;
// DSP BLOCK ///////////////////////////////////////////////////////////////////	
	while ( n-- )										// while a sample remains, do dsp ( n-- is the counter )
	{  
		x->sample[2] = x->sample[1];
		x->sample[1] = x->sample[0];
		x->sample[0] = *in++;							// *in++/*out++ should only be done once per while loop cycle
		
		x->spike = 0;
		
		if( x->sample[1] > 0 )
		{
			if( ( x->sample[0] < x->sample[1] ) && ( x->sample[1] > x->sample[2] ) )
			{
				x->spike = x->sample[1];
			}
		}
		if( x->sample[1] < 0 )
		{
			if( ( x->sample[0] > x->sample[1] ) && ( x->sample[1] < x->sample[2] ) )
			{
				x->spike = x->sample[1];
			}
		}
		*out++ = x->spike;								// sample is used to make things clearer
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 5);										// always return w plus one more than the 2nd argument in dsp_add() 
} 

// spiker_assist -----------------------------------------------------------------
void spiker_assist(t_spiker *x, void *b, long m, long a, char *s)
{
	switch (m)											// inlet / outlet switch
	{
		case 1:											// 1 = inlets
			switch (a)									// inlet number switch
			{
				case 0:									// inlet 0
					sprintf(s, "(signal) Thru~ Input");
					break;								// assist message
				default:
					break;
			}
			break;
		case 2:											// 2 = outlets
			switch (a)									// outlet number switch
			{
				case 0:									// outlet 0
					sprintf(s, "(signal) Thru~ Output");
					break;								// assist message
				default:
					break;
			}
			break;
	}
}

// spiker_new() ------------------------------------------------------------------ 
void *spiker_new(void)									// spiker~ 'constructor'
{ 
	t_spiker *x = (t_spiker *)newobject(spiker_class);	// register the object with spiker and save its address as *x
	dsp_setup((t_pxobject *)x, 1);						// create left inlet 
	outlet_new((t_pxobject *)x, "signal");				// create left outlet 
	
	x->sample[0] = 0;									// init x->samples
	x->sample[1] = 0;
	x->sample[2] = 0;
	
	return (x);											// where am i?
} 

// spiker_free() -----------------------------------------------------------------
void spiker_free(t_spiker *x)							// spiker~ 'destructor'
{
	dsp_free((t_pxobject *)x);							// spiker free function
}
